!=========================================================================
! Copyright (C) GemTalk Systems 1986-2015.  All Rights Reserved.
!
! $Id: deplistbucket.gs,v 1.7 2008-01-09 19:02:32 stever Exp $
!
! Superclass Hierarchy:
!   DepListBucket, Array, SequenceableCollection, Collection, Object.
!
! class created in idxclasses.topaz
!=========================================================================

removeallmethods DepListBucket
removeallclassmethods DepListBucket
protectmethods

category: 'For Documentation Installation only'
classmethod: DepListBucket
installDocumentation

< unprotected >

| doc txt |
doc := GsClassDocumentation _newForPrivateGsClass: self.

txt := (GsDocText new) details:
'A DepListBucket is a collision bucket for a DepListTable.' .
doc documentClassWith: txt.

txt := (GsDocText new) details:
'The number of dependency lists in the instance.'.
doc documentInstVar: #numElements with: txt.

self description: doc.
%

!==========================================================================
! Below this point: only method definitions in fileout order
!==========================================================================


! ------------------- Class methods for DepListBucket
category: 'Instance Creation'
classmethod: DepListBucket
new

"Returns a DepListBucket with a default capacity of ten entries."
<protected>

^ self new: 3

%
category: 'Instance Creation'
classmethod: DepListBucket
new: aSize

"Returns a DepListBucket with the given capacity of entries."
<protected>

^ (super new: aSize) initialize

%
! ------------------- Instance methods for DepListBucket
category: 'Adding'
method: DepListBucket
add: newObject

"Protected version of Array | add:."

<protected primitive: 203>
self _primitiveFailed: #add: .
self _uncontinuableError

%
category: 'Adding'
method: DepListBucket
addLast: newObject

"Protected version of Array | addLast:."

<protected primitive: 203>
self _primitiveFailed: #addLast: .
self _uncontinuableError

%
category: 'Updating'
method: DepListBucket
at: depList

"Returns the dependency list in the receiver that is equivalent to the given
 dependency list.  If one does not exist, add the given dependency list to the
 receiver and returns the given dependency list.  If one is found, the given
 dependency list is sized to zero."

<protected primitive: 215>
" primitive will fail if the receiver is a large object; do the work in
  GemStone Smalltalk "
| dl lowPt highPt i |
numElements == 0
  ifTrue: [
    numElements := 1.
    self _at: 1 put: depList.
    depList assignToSegment: self segment.
    ^ depList
  ].

" binary search "
lowPt := 1.
highPt := numElements.
[ lowPt <= highPt ] whileTrue: [
  i := (lowPt + highPt) quo: 2.
  dl := self _rcAt: i.

  dl = depList
    ifTrue: [
      depList size: 0.
      ^ dl
    ].

  depList < dl
    ifTrue: [ highPt := i - 1 ]
    ifFalse: [ lowPt := i + 1 ]
].

depList > (self _rcAt: i)
  ifTrue: [ i := i + 1 ].

" if inserting on the end "
i > numElements
  ifTrue: [
    " see if need to grow the object "
    i > self _basicSize
      ifTrue: [ " make bigger for insertion "
        self size: self _basicSize + self growIncrement
      ]
  ]
  ifFalse: [ " inserting in the middle, make room for new entry "
    self copyFrom: i to: numElements into: self startingAt: i + 1.
  ].

numElements := numElements + 1.
self _at: i put: depList.
depList assignToSegment: self segment.
^ depList

%
category: 'Updating'
method: DepListBucket
at: anIndex put: aValue

"Protected version of Object | at:put:."

<protected primitive: 268>

(anIndex _isSmallInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
(self class isBytes)
ifTrue: [ ((aValue class ~~ SmallInteger) _or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [^ aValue _error: #rtErrExpectedByteValue].
  ].

self _primitiveFailed: #at:put: .
self _uncontinuableError

%
category: 'Updating'
method: DepListBucket
basicAt: anIndex put: aValue

"Protected version of Object | basicAt:put:."
<protected>

^self _basicAt: anIndex put: aValue

%
category: 'Copying'
method: DepListBucket
copyFrom: index1 to: index2 into: aSeqColl startingAt: destIndex

"Protected version of Array | copyFrom:to:into:startingAt:."

<protected primitive: 808"with rcvrClass">

(index1 > index2)
  ifTrue:[ index1 _error: #rtErrBadCopyFromTo args: #[index2]].

(index1 < 1) ifTrue:[ self _errorIndexOutOfRange: index1].

(index2 > self size) ifTrue:[ self _errorIndexOutOfRange: index2].

((destIndex < 1) _or: [(destIndex > (aSeqColl size + 1))])
  ifTrue:[ self _errorIndexOutOfRange: destIndex].
^ super copyFrom: index1 to: index2 into: aSeqColl startingAt: destIndex

%
category: 'Enumerating'
method: DepListBucket
do: aBlock

"For each dependency list in the receiver, evaluates the one-argument block
 aBlock with the dependency list as the argument."
<protected>

| aDL |
1 to: numElements do: [ :j |
  aDL := self _at: j.
  aDL == nil
    ifFalse: [ aBlock value: aDL ]
]

%
category: 'Constants'
method: DepListBucket
growIncrement

"Returns the number of entries to grow when needed."
<protected>

^ 10

%
category: 'Initializing'
method: DepListBucket
initialize

"Initializes the instance variable of the receiver."
<protected>

numElements := 0.

%
category: 'Copying'
method: DepListBucket
insertAll: aSeqColl at: anIndex

"Protected version of Array | insertAll:at:."

<protected primitive: 806"with rcvrClass">

^ super insertAll: aSeqColl at: anIndex

%
category: 'Updating'
method: DepListBucket
instVarAt: anIndex put: aValue

"Protected version of Object | instVarAt:put:."

<protected primitive: 804"with rcvrClass">
(self class allInstVarNames size == 0)
  ifTrue: [ ^ self _error: #rtErrNoInstVars].
(anIndex _isInteger)
  ifTrue: [((anIndex < 1) | (anIndex > self class allInstVarNames size))
        ifTrue: [ ^ self _errorIndexOutOfRange: anIndex]] "out of bounds"
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].

self _primitiveFailed: #instVarAt:put: .
self _uncontinuableError

%
category: 'Removing'
method: DepListBucket
remove: aDepList

"Remove the given dependency list from the receiver.  Returns whether the
 removal occurred."
<protected>

| offset |
offset := self _linearSearchFor: aDepList.
offset == 0
  ifTrue: [ ^ false ].

self _deleteNoShrinkFrom: offset to: offset .
numElements := numElements - 1.

^ true

%
category: 'Updating'
method: DepListBucket
reset

"Reset all entries to nil and the number of elements to zero."
<protected>

super size: 0.
numElements := 0.
super size: self growIncrement.
System _addEntireObjectToRcReadSet: self

%
category: 'Updating'
method: DepListBucket
resizeIfNecessary

"Check the size of the receiver to see if it is unnecessarily greater
 than a page in size.  If so, resize it."
<protected>

( self _basicSize > 2000 _and: [ numElements < 2000 ] )
  ifTrue: [ super size: (2000 min: numElements + self growIncrement) ].

%
category: 'Accessing'
method: DepListBucket
size

"Returns the value of the numElements instance variable."
<unprotected>

^ numElements

%
category: 'Updating'
method: DepListBucket
size: anInteger

"Protected version of Object | size:."

<protected primitive: 803"with rcvrClass">

(self class isIndexable not) "not an indexable object"
  ifTrue: [ ^ self _errorNotIndexable].
(anInteger _isInteger)
   ifTrue: [ ^ self _errorIndexOutOfRange: anInteger]
   ifFalse: [ ^ self _errorNonIntegerIndex: anInteger].
self _primitiveFailed: #size: .
self _uncontinuableError

%
category: 'Adding'
method: DepListBucket
_addAllFromNsc: aBag

"Protected version of Array | _addAllFromNsc:."

<protected primitive: 202>
self _primitiveFailed: #_addAllFromNsc: .
self _uncontinuableError

%
category: 'Updating'
method: DepListBucket
_at: anIndex put: aValue

"Protected version of Object | _at:put:."

<protected primitive: 268>

(anIndex _isSmallInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
(self class isBytes)
ifTrue:
  [ ((aValue class ~~ SmallInteger) _or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [^ aValue _error: #rtErrExpectedByteValue].
  ].

self _primitiveFailed: #_at:put: .
self _uncontinuableError

%
category: 'Updating'
method: DepListBucket
_basicAt: anIndex put: aValue

"Protected version of Object | _basicAt:put:."

<protected primitive: 268>

(anIndex _isSmallInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
(self class isIndexable) "not an indexable object"
  ifFalse: [ ^ self _errorNotIndexable].
((anIndex < 1) | (anIndex > (self size + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
(self class isBytes)
ifTrue:
  [ ((aValue class ~~ SmallInteger) _or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [^ aValue _error: #rtErrExpectedByteValue].
  ].

self _primitiveFailed: #_basicAt:put: .
self _uncontinuableError

%
category: 'Updating'
method: DepListBucket
_basicSize: anInteger

"Protected version of Object | _basicSize:."

<protected primitive: 803"with rcvrClass">

(self class isIndexable not) "not an indexable object"
  ifTrue: [ ^ self _errorNotIndexable].
(anInteger _isInteger)
   ifTrue: [ ^ self _errorIndexOutOfRange: anInteger]
   ifFalse: [ ^ self _errorNonIntegerIndex: anInteger].
self _primitiveFailed: #size: .
self _uncontinuableError

%
category: 'Searching'
method: DepListBucket
_binarySearchFor: aDepList

"Returns the offset into the receiver where the given dependency list
 can be found.  If not found, returns zero."
<protected>

| lowPt midPt highPt currDL |

numElements == 0
  ifTrue: [ ^ 0 ].

lowPt := 1.
highPt := numElements.
[ lowPt <= highPt ] whileTrue: [
  midPt := (lowPt + highPt) quo: 2.
  currDL := self _rcAt: midPt.

  "use identity-based comparison of elements of dependency lists"
  (currDL hasIdenticalContents: aDepList)
    ifTrue: [ ^ midPt ].

  aDepList < currDL
    ifTrue: [ highPt := midPt - 1 ]
    ifFalse: [ lowPt := midPt + 1 ]
].

^ 0

%
category: 'Auditing'
method: DepListBucket
_hasEntryOutOfOrder

"Return whether the receiver has any entry out of order."

<protected>

| depList prevDepList |
self size > 1
  ifTrue: [
    prevDepList := self _at: 1.
    2 to: self size do: [ :j |
      depList := self _at: j.
      depList < prevDepList
        ifTrue: [ ^ true ]
    ]
  ].
^ false

%
category: 'Searching'
method: DepListBucket
_linearSearchFor: aDepList

"Private.  Returns the offset into the receiver where the given dependency list
 can be found.  If not found, returns zero."

<protected>

1 to: numElements do: [ :j |
  aDepList == (self _at: j)
    ifTrue: [ ^ j ]
].
^ 0

%
category: 'Updating'
method: DepListBucket
_primitiveAt: anIndex put: aValue

"Protected version of Object | _primitiveAt:put:."

<protected primitive: 281>

(anIndex _isSmallInteger)
  ifFalse: [ ^ self _errorNonIntegerIndex: anIndex].
((anIndex < 1) | (anIndex > (self _primitiveSize + 1))) "out of bounds"
  ifTrue: [ ^ self _errorIndexOutOfRange: anIndex].
(self class isBytes)
ifTrue:
  [ ((aValue class ~~ SmallInteger) _or: [ (aValue < 0) | (aValue > 255) ])
    ifTrue: [^ aValue _error: #rtErrExpectedByteValue].
  ].

self _primitiveFailed: #_primitiveAt:put: .
self _uncontinuableError

%


unprotectmethods

